#include <amxmodx>
#include <fakemeta>
#include <reapi>

#define SMOKE_DURATION 15.0
#define SMOKE_OPAQUE 255.0  
#define SPRITES_COUNT 10
#define SMOKE_RADIUS 30.0

//#define WATER_FIX

new const SMOKE_CLASS[ ] = "custom_smoke"
new const PARTS_CLASS[ ] = "custom_smoke_parts"
new const SMOKE_SPRITE[] = "sprites/smoke_grey_parable.spr"
new const Float:SMOKE_COLOR[3] = {255.0, 255.0, 255.0}

public plugin_init() 
{
	register_plugin("Another custom smoke", "1.0", "fork by MurLemur")
	RegisterHookChain( RG_CGrenade_ExplodeSmokeGrenade, "OnExplodeSmokeGrenade_Pre" )
	RegisterHookChain( RG_CSGameRules_CleanUpMap, "OnCleanUpMap_Pre" )
	
	#if defined WATER_FIX
	changeAllWaterRendermode(kRenderNormal)
	#endif
}	

public plugin_precache() 
{
	if( !file_exists( SMOKE_SPRITE ) )
		set_fail_state( "Can't find '%s'", SMOKE_SPRITE )

	precache_model(SMOKE_SPRITE)
	force_unmodified( force_exactfile, { 0, 0, 0 }, { 0, 0, 0 }, SMOKE_SPRITE )
	
}

public OnExplodeSmokeGrenade_Pre( pEnt )
{
	if( !is_entity( pEnt ) )
		return HC_CONTINUE

	new Float:vOrigin[ 3 ]
	get_entvar( pEnt, var_origin, vOrigin )

	emit_sound( pEnt, CHAN_WEAPON, "weapons/sg_explode.wav", VOL_NORM, ATTN_NORM, 0, PITCH_NORM )

	set_entvar( pEnt, var_flags, FL_KILLME )

 	new pSmokeEnt = rg_create_entity( "info_target", .useHashTable = false )

	if( !pSmokeEnt )
		return HC_SUPERCEDE

	set_entvar( pSmokeEnt, var_classname, SMOKE_CLASS )
	set_entvar( pSmokeEnt, var_nextthink, get_gametime( ) )
	set_entvar( pSmokeEnt, var_origin, vOrigin )
	set_entvar( pSmokeEnt, var_animtime, SMOKE_DURATION )
	SetThink( pSmokeEnt, "Smoke_Think_Pre" ) 
	
	vOrigin[2] += 70
	for (new x=0; x < SPRITES_COUNT; x++) {
		AttachSprite(pSmokeEnt,vOrigin)
	}
	
	return HC_SUPERCEDE
}

public Smoke_Think_Pre( pEnt )
{
	if( !is_entity( pEnt ) )
		return 

	new Float:fFreq = 1.0

	new Float:flTime = Float:get_entvar( pEnt, var_animtime ) - fFreq

	if( flTime > 0.0 )
	{
		set_entvar( pEnt, var_nextthink, get_gametime( ) + fFreq )
		set_entvar( pEnt, var_animtime, flTime )
	}
	else
	{
		new iEnt = NULLENT
		
		while((iEnt = rg_find_ent_by_class(iEnt, PARTS_CLASS)) > 0) 
			if (get_entvar(iEnt, var_owner) == pEnt)
				set_entvar(iEnt, var_renderamt, SMOKE_OPAQUE - 5)
	
		set_entvar( pEnt, var_flags, FL_KILLME )
	}	
}

public OnCleanUpMap_Pre( )
{
 	new pEnt = NULLENT
	
	while((pEnt = rg_find_ent_by_class(pEnt, SMOKE_CLASS)) > 0) 
		set_entvar(pEnt, var_flags, FL_KILLME)
	
	pEnt = NULLENT
	
	while((pEnt = rg_find_ent_by_class(pEnt, PARTS_CLASS)) > 0) 
		set_entvar(pEnt, var_flags, FL_KILLME)
	
}

AttachSprite(iEntity,Float:vecOrigin[3]) {
		new iEnt = rg_create_entity("env_sprite")

		if(is_nullent(iEnt))
			return 
		
		engfunc(EngFunc_SetModel, iEnt, SMOKE_SPRITE)
		vecOrigin[0] += random_float(-SMOKE_RADIUS, SMOKE_RADIUS)
		vecOrigin[1] += random_float(-SMOKE_RADIUS, SMOKE_RADIUS)
		set_entvar(iEnt, var_origin, vecOrigin)
		set_entvar(iEnt, var_classname, PARTS_CLASS )
		set_entvar(iEnt, var_owner, iEntity)
		set_entvar(iEnt, var_movetype, MOVETYPE_NOCLIP)
		rg_set_rendering(iEnt,kRenderFxNone,SMOKE_COLOR,kRenderTransAlpha,SMOKE_OPAQUE)	
		set_entvar(iEnt, var_scale, 1.1)

		SetThink(iEnt, "Part_Think")
		set_entvar(iEnt, var_nextthink, get_gametime() + 0.5)	
}

public Part_Think(const iEnt) {
	
	new Float: fRenderAmt
	get_entvar(iEnt, var_renderamt, fRenderAmt)
	
	if(is_nullent(get_entvar(iEnt, var_owner)) || fRenderAmt < SMOKE_OPAQUE)
	{	
		if(fRenderAmt > 30.0)
		{
			set_entvar(iEnt, var_renderamt, fRenderAmt - 30.0)
			set_entvar(iEnt, var_nextthink, get_gametime() + 0.1)
		}
		else 
			set_entvar(iEnt, var_flags, FL_KILLME)
		return
	}
	
	set_entvar(iEnt, var_nextthink, get_gametime() + 0.5)
}

stock rg_set_rendering(const id, const fx = kRenderFxNone, const Float:fColor[3] = {0.0, 0.0, 0.0}, const render = kRenderNormal, const Float:fAmount = 0.0) 
{
	set_entvar(id, var_renderfx, fx)
	set_entvar(id, var_rendercolor, fColor)
	set_entvar(id, var_rendermode, render)
	set_entvar(id, var_renderamt, fAmount)
}

#if defined WATER_FIX
changeAllWaterRendermode(const mode = kRenderNormal) {
  new ent = -1
  while((ent = rg_find_ent_by_class(ent, "func_water"))) {
    set_pev(ent, pev_rendermode, mode)
  }
}
#endif